@@ -25,7 +25,6 @@
 #define GDOME_H
 
 #include <glib.h>
-#include <libxml/tree.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,6 +32,7 @@ extern "C" {
 
 /* error types for our exception handling system. */
 typedef enum {
+  GDOME_NOEXCEPTION_ERR = 0,
 	GDOME_INDEX_SIZE_ERR = 1,
 	GDOME_DOMSTRING_SIZE_ERR = 2,
 	GDOME_HIERARCHY_REQUEST_ERR = 3,
@@ -51,6 +51,21 @@ typedef enum {
 	GDOME_NULL_POINTER_ERR = 100
 } GdomeExceptionCode;
 
+typedef enum {
+	GDOME_CORE_EXCEPTION = 0 << 16,
+	GDOME_EVENT_EXCEPTION = 1 << 16,
+	GDOME_XPATH_EXCEPTION = 2 << 16
+} GdomeExceptionType;
+
+typedef enum {
+	GDOME_EXCEPTION_TYPE_MASK = 0x00FF0000,
+	GDOME_EXCEPTION_CODE_MASK = 0x0000FFFF
+} GdomeExceptionMasks;
+
+/* Exception Handling Macros */
+#define GDOME_EXCEPTION_TYPE(exc) ((exc) & GDOME_EXCEPTION_TYPE_MASK)
+#define GDOME_EXCEPTION_CODE(exc) ((exc) & GDOME_EXCEPTION_CODE_MASK)
+
 /* access type for node handles */
 typedef enum {
 	GDOME_READONLY_NODE = 0,
@@ -60,8 +75,9 @@ typedef enum {
 typedef enum {
 	GDOME_LOAD_PARSING = 0,
 	GDOME_LOAD_VALIDATING = 1,
-  GDOME_LOAD_RECOVERING = 2,
-  GDOME_LOAD_SUBSTITUTE_ENTITIES = 4
+	GDOME_LOAD_RECOVERING = 2,
+	GDOME_LOAD_SUBSTITUTE_ENTITIES = 4,
+	GDOME_LOAD_COMPLETE_ATTRS = 8
 } GdomeLoadingCode;
 
 typedef enum {
@@ -71,7 +87,7 @@ typedef enum {
 
 typedef struct _GdomeDOMString GdomeDOMString;
 typedef int GdomeBoolean;
-typedef unsigned short GdomeException;
+typedef guint32 GdomeException;
 typedef long long GdomeDOMTimeStamp;
 
 struct _GdomeDOMString {
@@ -99,13 +115,24 @@ typedef struct _GdomeNodeList GdomeNodeList;
 typedef struct _GdomeNotation GdomeNotation;
 typedef struct _GdomeProcessingInstruction GdomeProcessingInstruction;
 
-/* Events interfaces*/
+/* Events interfaces */
 typedef struct _GdomeEvent GdomeEvent;
 typedef struct _GdomeEventListener GdomeEventListener;
 typedef struct _GdomeMutationEvent GdomeMutationEvent;
 typedef struct _GnomeNode GnomeEventTarget;
 typedef struct _GnomeDocument GnomeDocumentEvent;
 
+/* XPath interfaces */
+typedef struct _GdomeXPathEvaluator GdomeXPathEvaluator;
+typedef struct _GdomeXPathNamespace GdomeXPathNamespace;
+typedef struct _GdomeXPathNSResolver GdomeXPathNSResolver;
+typedef struct _GdomeXPathResult GdomeXPathResult;
+
+/* Traversal interfaces */
+typedef struct _GdomeNodeFilter GdomeNodeFilter;
+typedef struct _GdomeNodeIterator GdomeNodeIterator;
+typedef struct _GdomeTreeWalker GdomeTreeWalker;
+
 typedef struct _GdomeNodeVtab GdomeNodeVtab;
 typedef struct _GdomeAttrVtab GdomeAttrVtab;
 typedef struct _GdomeCharacterDataVtab GdomeCharacterDataVtab;
@@ -123,6 +150,7 @@ typedef struct _GdomeNamedNodeMapVtab GdomeNamedNodeMapVtab;
 typedef struct _GdomeNodeListVtab GdomeNodeListVtab;
 typedef struct _GdomeNotationVtab GdomeNotationVtab;
 typedef struct _GdomeProcessingInstructionVtab GdomeProcessingInstructionVtab;
+typedef struct _GdomeXPathNamespaceVtab GdomeXPathNamespaceVtab;
 
 typedef enum {
 	GDOME_ELEMENT_NODE = 1,
@@ -136,7 +164,8 @@ typedef enum {
 	GDOME_DOCUMENT_NODE = 9,
 	GDOME_DOCUMENT_TYPE_NODE = 10,
 	GDOME_DOCUMENT_FRAGMENT_NODE = 11,
-	GDOME_NOTATION_NODE = 12
+	GDOME_NOTATION_NODE = 12,
+	GDOME_XPATH_NAMESPACE_NODE = 13
 } GdomeNodeType;
 
 typedef struct _GdomeEntitiesTableEntry GdomeEntitiesTableEntry;
@@ -148,87 +177,74 @@ struct _GdomeEntitiesTableEntry {
 };
 
 struct _GdomeNode {
-	const GdomeNodeVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeAttr {
-	const GdomeAttrVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeCharacterData {
-	const GdomeCharacterDataVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeText {
-	const GdomeTextVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeCDATASection {
-	const GdomeCDATASectionVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeComment {
-	const GdomeCommentVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeDOMImplementation {
-	const GdomeDOMImplementationVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeDocument {
-	const GdomeDocumentVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeDocumentFragment {
-	const GdomeDocumentFragmentVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeDocumentType {
-	const GdomeDocumentTypeVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeElement {
-	const GdomeElementVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeEntity {
-	const GdomeEntityVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeEntityReference {
-	const GdomeEntityReferenceVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeNamedNodeMap {
-	const GdomeNamedNodeMapVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeNodeList {
-	const GdomeNodeListVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeNotation {
-	const GdomeNotationVtab *vtab; /* deprecated */
 	gpointer user_data;
 };
 
 struct _GdomeProcessingInstruction {
-	const GdomeProcessingInstructionVtab *vtab; /* deprecated */
+	gpointer user_data;
+};
+
+struct _GdomeXPathNamespace {
 	gpointer user_data;
 };
 
@@ -602,12 +618,12 @@ GdomeBoolean          gdome_a_canAppend            (GdomeAttr *self,
 
 GdomeDOMString *gdome_cd_data (GdomeCharacterData *self, GdomeException *exc);
 void gdome_cd_set_data (GdomeCharacterData *self, GdomeDOMString *data, GdomeException *exc);
-unsigned long gdome_cd_length (GdomeCharacterData *self, GdomeException *exc);
-GdomeDOMString *gdome_cd_substringData (GdomeCharacterData *self, unsigned long offset, unsigned long count, GdomeException *exc);
+gulong gdome_cd_length (GdomeCharacterData *self, GdomeException *exc);
+GdomeDOMString *gdome_cd_substringData (GdomeCharacterData *self, gulong offset, gulong count, GdomeException *exc);
 void gdome_cd_appendData (GdomeCharacterData *self, GdomeDOMString *arg, GdomeException *exc);
-void gdome_cd_insertData (GdomeCharacterData *self, unsigned long offset, GdomeDOMString *arg, GdomeException *exc);
-void gdome_cd_deleteData (GdomeCharacterData *self, unsigned long offset, unsigned long count, GdomeException *exc);
-void gdome_cd_replaceData (GdomeCharacterData *self, unsigned long offset, unsigned long count, GdomeDOMString *arg, GdomeException *exc);
+void gdome_cd_insertData (GdomeCharacterData *self, gulong offset, GdomeDOMString *arg, GdomeException *exc);
+void gdome_cd_deleteData (GdomeCharacterData *self, gulong offset, gulong count, GdomeException *exc);
+void gdome_cd_replaceData (GdomeCharacterData *self, gulong offset, gulong count, GdomeDOMString *arg, GdomeException *exc);
 void gdome_cd_ref (GdomeCharacterData *self, GdomeException *exc);
 void gdome_cd_unref (GdomeCharacterData *self, GdomeException *exc);
 gpointer gdome_cd_query_interface (GdomeCharacterData *self, const char *interface, GdomeException *exc);
@@ -644,15 +660,15 @@ GdomeBoolean gdome_cd_canAppend (GdomeCharacterData *self, GdomeNode *newChild,
 
 
 
-GdomeText *gdome_t_splitText (GdomeText *self, unsigned long offset, GdomeException *exc);
+GdomeText *gdome_t_splitText (GdomeText *self, gulong offset, GdomeException *exc);
 GdomeDOMString *gdome_t_data (GdomeText *self, GdomeException *exc);
 void gdome_t_set_data (GdomeText *self, GdomeDOMString *data, GdomeException *exc);
-unsigned long gdome_t_length (GdomeText *self, GdomeException *exc);
-GdomeDOMString *gdome_t_substringData (GdomeText *self, unsigned long offset, unsigned long count, GdomeException *exc);
+gulong gdome_t_length (GdomeText *self, GdomeException *exc);
+GdomeDOMString *gdome_t_substringData (GdomeText *self, gulong offset, gulong count, GdomeException *exc);
 void gdome_t_appendData (GdomeText *self, GdomeDOMString *arg, GdomeException *exc);
-void gdome_t_insertData (GdomeText *self, unsigned long offset, GdomeDOMString *arg, GdomeException *exc);
-void gdome_t_deleteData (GdomeText *self, unsigned long offset, unsigned long count, GdomeException *exc);
-void gdome_t_replaceData (GdomeText *self, unsigned long offset, unsigned long count, GdomeDOMString *arg, GdomeException *exc);
+void gdome_t_insertData (GdomeText *self, gulong offset, GdomeDOMString *arg, GdomeException *exc);
+void gdome_t_deleteData (GdomeText *self, gulong offset, gulong count, GdomeException *exc);
+void gdome_t_replaceData (GdomeText *self, gulong offset, gulong count, GdomeDOMString *arg, GdomeException *exc);
 void gdome_t_ref (GdomeText *self, GdomeException *exc);
 void gdome_t_unref (GdomeText *self, GdomeException *exc);
 gpointer gdome_t_query_interface (GdomeText *self, const char *interface, GdomeException *exc);
@@ -689,15 +705,15 @@ GdomeBoolean gdome_t_canAppend (GdomeText *self, GdomeNode *newChild, GdomeExcep
 
 
 
-GdomeText *gdome_cds_splitText (GdomeCDATASection *self, unsigned long offset, GdomeException *exc);
+GdomeText *gdome_cds_splitText (GdomeCDATASection *self, gulong offset, GdomeException *exc);
 GdomeDOMString *gdome_cds_data (GdomeCDATASection *self, GdomeException *exc);
 void gdome_cds_set_data (GdomeCDATASection *self, GdomeDOMString *data, GdomeException *exc);
-unsigned long gdome_cds_length (GdomeCDATASection *self, GdomeException *exc);
-GdomeDOMString *gdome_cds_substringData (GdomeCDATASection *self, unsigned long offset, unsigned long count, GdomeException *exc);
+gulong gdome_cds_length (GdomeCDATASection *self, GdomeException *exc);
+GdomeDOMString *gdome_cds_substringData (GdomeCDATASection *self, gulong offset, gulong count, GdomeException *exc);
 void gdome_cds_appendData (GdomeCDATASection *self, GdomeDOMString *arg, GdomeException *exc);
-void gdome_cds_insertData (GdomeCDATASection *self, unsigned long offset, GdomeDOMString *arg, GdomeException *exc);
-void gdome_cds_deleteData (GdomeCDATASection *self, unsigned long offset, unsigned long count, GdomeException *exc);
-void gdome_cds_replaceData (GdomeCDATASection *self, unsigned long offset, unsigned long count, GdomeDOMString *arg, GdomeException *exc);
+void gdome_cds_insertData (GdomeCDATASection *self, gulong offset, GdomeDOMString *arg, GdomeException *exc);
+void gdome_cds_deleteData (GdomeCDATASection *self, gulong offset, gulong count, GdomeException *exc);
+void gdome_cds_replaceData (GdomeCDATASection *self, gulong offset, gulong count, GdomeDOMString *arg, GdomeException *exc);
 void gdome_cds_ref (GdomeCDATASection *self, GdomeException *exc);
 void gdome_cds_unref (GdomeCDATASection *self, GdomeException *exc);
 gpointer gdome_cds_query_interface (GdomeCDATASection *self, const char *interface, GdomeException *exc);
@@ -736,12 +752,12 @@ GdomeBoolean gdome_cds_canAppend (GdomeCDATASection *self, GdomeNode *newChild,
 
 GdomeDOMString *gdome_c_data (GdomeComment *self, GdomeException *exc);
 void gdome_c_set_data (GdomeComment *self, GdomeDOMString *data, GdomeException *exc);
-unsigned long gdome_c_length (GdomeComment *self, GdomeException *exc);
-GdomeDOMString *gdome_c_substringData (GdomeComment *self, unsigned long offset, unsigned long count, GdomeException *exc);
+gulong gdome_c_length (GdomeComment *self, GdomeException *exc);
+GdomeDOMString *gdome_c_substringData (GdomeComment *self, gulong offset, gulong count, GdomeException *exc);
 void gdome_c_appendData (GdomeComment *self, GdomeDOMString *arg, GdomeException *exc);
-void gdome_c_insertData (GdomeComment *self, unsigned long offset, GdomeDOMString *arg, GdomeException *exc);
-void gdome_c_deleteData (GdomeComment *self, unsigned long offset, unsigned long count, GdomeException *exc);
-void gdome_c_replaceData (GdomeComment *self, unsigned long offset, unsigned long count, GdomeDOMString *arg, GdomeException *exc);
+void gdome_c_insertData (GdomeComment *self, gulong offset, GdomeDOMString *arg, GdomeException *exc);
+void gdome_c_deleteData (GdomeComment *self, gulong offset, gulong count, GdomeException *exc);
+void gdome_c_replaceData (GdomeComment *self, gulong offset, gulong count, GdomeDOMString *arg, GdomeException *exc);
 void gdome_c_ref (GdomeComment *self, GdomeException *exc);
 void gdome_c_unref (GdomeComment *self, GdomeException *exc);
 gpointer gdome_c_query_interface (GdomeComment *self, const char *interface, GdomeException *exc);
@@ -785,17 +801,15 @@ gpointer gdome_di_query_interface (GdomeDOMImplementation *self, const char *int
 GdomeBoolean gdome_di_hasFeature (GdomeDOMImplementation *self, GdomeDOMString *feature, GdomeDOMString *version, GdomeException *exc);
 GdomeDocumentType *gdome_di_createDocumentType (GdomeDOMImplementation *self, GdomeDOMString *qualifiedName, GdomeDOMString *publicId, GdomeDOMString *systemId, GdomeException *exc);
 GdomeDocument *gdome_di_createDocument (GdomeDOMImplementation *self, GdomeDOMString *namespaceURI, GdomeDOMString *qualifiedName, GdomeDocumentType *doctype, GdomeException *exc);
-GdomeDocument *gdome_di_parseFile (GdomeDOMImplementation *self, const char *uri, GdomeException *exc);
-GdomeDocument *gdome_di_validateFile (GdomeDOMImplementation *self, const char *uri, GdomeException *exc);
-GdomeBoolean gdome_di_saveFile (GdomeDOMImplementation *self, const char *filename, GdomeDocument *doc, GdomeException *exc);
 void gdome_di_freeDoc (GdomeDOMImplementation *self, GdomeDocument *doc, GdomeException *exc);
-GdomeBoolean gdome_di_saveFormatFile (GdomeDOMImplementation *self, const char *filename, GdomeDocument *doc, int format, GdomeException *exc);
 GdomeDocument *gdome_di_createDocFromURI (GdomeDOMImplementation *self, const char *uri, unsigned int mode, GdomeException *exc);
 GdomeDocument *gdome_di_createDocFromMemory (GdomeDOMImplementation *self, char *buffer, unsigned int mode, GdomeException *exc);
 GdomeDocument *gdome_di_createDocFromURIWithEntitiesTable (GdomeDOMImplementation *self, const char *uri, const GdomeEntitiesTableEntry entityTable[], unsigned int mode, GdomeException *exc);
 GdomeDocument *gdome_di_createDocFromMemoryWithEntitiesTable (GdomeDOMImplementation *self, char *buffer, const GdomeEntitiesTableEntry entityTable[], unsigned int mode, GdomeException *exc);
 GdomeBoolean gdome_di_saveDocToFile (GdomeDOMImplementation *self, GdomeDocument *doc, const char *filename, GdomeSavingCode mode, GdomeException *exc);
+GdomeBoolean gdome_di_saveDocToFileEnc (GdomeDOMImplementation *self, GdomeDocument *doc, const char *filename, const char *encoding, GdomeSavingCode mode, GdomeException *exc);
 GdomeBoolean gdome_di_saveDocToMemory (GdomeDOMImplementation *self, GdomeDocument *doc, char **mem, GdomeSavingCode mode, GdomeException *exc);
+GdomeBoolean gdome_di_saveDocToMemoryEnc (GdomeDOMImplementation *self, GdomeDocument *doc, char **mem, const char *encoding, GdomeSavingCode mode, GdomeException *exc);
 
 
 GdomeDocumentType *gdome_doc_doctype (GdomeDocument *self, GdomeException *exc);
@@ -1014,8 +1028,8 @@ gpointer gdome_nnm_query_interface (GdomeNamedNodeMap *self, const char *interfa
 GdomeNode *gdome_nnm_getNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name, GdomeException *exc);
 GdomeNode *gdome_nnm_setNamedItem (GdomeNamedNodeMap *self, GdomeNode *arg, GdomeException *exc);
 GdomeNode *gdome_nnm_removeNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name, GdomeException *exc);
-GdomeNode *gdome_nnm_item (GdomeNamedNodeMap *self, unsigned long index, GdomeException *exc);
-unsigned long gdome_nnm_length (GdomeNamedNodeMap *self, GdomeException *exc);
+GdomeNode *gdome_nnm_item (GdomeNamedNodeMap *self, gulong index, GdomeException *exc);
+gulong gdome_nnm_length (GdomeNamedNodeMap *self, GdomeException *exc);
 GdomeNode *gdome_nnm_getNamedItemNS (GdomeNamedNodeMap *self, GdomeDOMString *namespaceURI, GdomeDOMString *localName, GdomeException *exc);
 GdomeNode *gdome_nnm_setNamedItemNS (GdomeNamedNodeMap *self, GdomeNode *arg, GdomeException *exc);
 GdomeNode *gdome_nnm_removeNamedItemNS (GdomeNamedNodeMap *self, GdomeDOMString *namespaceURI, GdomeDOMString *localName, GdomeException *exc);
@@ -1025,8 +1039,8 @@ GdomeNode *gdome_nnm_removeNamedItemNS (GdomeNamedNodeMap *self, GdomeDOMString
 void gdome_nl_ref (GdomeNodeList *self, GdomeException *exc);
 void gdome_nl_unref (GdomeNodeList *self, GdomeException *exc);
 gpointer gdome_nl_query_interface (GdomeNodeList *self, const char *interface, GdomeException *exc);
-GdomeNode *gdome_nl_item (GdomeNodeList *self, unsigned long index, GdomeException *exc);
-unsigned long gdome_nl_length (GdomeNodeList *self, GdomeException *exc);
+GdomeNode *gdome_nl_item (GdomeNodeList *self, gulong index, GdomeException *exc);
+gulong gdome_nl_length (GdomeNodeList *self, GdomeException *exc);
 
 
 
@@ -1104,6 +1118,18 @@ void gdome_pi_removeEventListener (GdomeProcessingInstruction *self, GdomeDOMStr
 GdomeBoolean gdome_pi_dispatchEvent (GdomeProcessingInstruction *self, GdomeEvent *evt, GdomeException *exc);
 void gdome_pi_subTreeDispatchEvent (GdomeProcessingInstruction *self, GdomeEvent *evt, GdomeException *exc);
 GdomeBoolean gdome_pi_canAppend (GdomeProcessingInstruction *self, GdomeNode *newChild, GdomeException *exc);
+GdomeElement * gdome_xpns_ownerElement (GdomeXPathNamespace *self, GdomeException *exc);
+GdomeDocument     * gdome_xpns_ownerDocument        (GdomeXPathNamespace *self,
+																										 GdomeException *exc);
+GdomeDOMString    * gdome_xpns_prefix               (GdomeXPathNamespace *self,
+																										 GdomeException *exc);
+GdomeDOMString    * gdome_xpns_nodeName             (GdomeXPathNamespace *self,
+																										 GdomeException *exc);
+unsigned short      gdome_xpns_nodeType             (GdomeXPathNamespace *self,
+																										 GdomeException *exc);
+GdomeDOMString    * gdome_xpns_namespaceURI         (GdomeXPathNamespace *self,
+																										 GdomeException *exc);
+
 
 /* Casting Macro */
 #include <gdome-util.h>
@@ -1213,10 +1239,14 @@ GdomeBoolean gdome_pi_canAppend (GdomeProcessingInstruction *self, GdomeNode *ne
  */
 #define GDOME_EVNTT(node) gdome_cast_evntt((GdomeNode *)node)
 
-/* FIXME: Does this belong here? */
-GdomeNode *         gdome_xml_n_mkref        (xmlNode *n);
-GdomeDocument *     gdome_xml_from_document  (xmlDoc *doc);
-xmlNode *           gdome_xml_n_get_xmlNode  (GdomeNode *node);
+/**
+ * GDOME_XMLNS_NAMESPACE:
+ *
+ * This is the namespace for the xmlns: prefix for namespace declaration
+ * attributes.
+ */
+#define GDOME_XMLNS_NAMESPACE \
+    (const xmlChar *) "http://www.w3.org/2000/xmlns/"
 
 #ifdef __cplusplus
 }
